@@ -123,9 +123,9 @@ class TestIndexThings:
 
     def test_pypi_index_attributes(self, mapp):
         mapp.login_root()
-        data = mapp.getjson("/root/pypi")
+        data = mapp.getjson("/root/pypi?no_projects=")
         res = data["result"]
-        res.pop("projects")
+        res.pop("projects", None)
         assert sorted(res.keys()) == sorted([
             "type", "volatile", "title", "mirror_url", "mirror_web_url_fmt"])
         assert res["type"] == "mirror"
@@ -410,9 +410,9 @@ class TestMirrorIndexThings:
         r = mapp.downloadrelease((404, 502), result[0])
         msg = r['message']
         assert (
-            'error 404 getting' in msg or
-            'received 404 from master' in msg or
-            'received 502 from master' in msg)
+            'error 404 getting' in msg
+            or 'received 404 from master' in msg
+            or 'received 502 from master' in msg)
 
     def test_download_release(self, mapp, simpypi):
         mapp.create_and_login_user('mirror6')
@@ -435,7 +435,7 @@ class TestMirrorIndexThings:
         indexconfig = dict(
             type="mirror",
             mirror_url=simpypi.simpleurl,
-            mirror_cache_expiry=1800)
+            mirror_cache_expiry=0)
         mapp.create_index("mirror", indexconfig=indexconfig)
         mapp.use("mirror7/mirror")
         simpypi.add_project('pkg')
@@ -443,8 +443,6 @@ class TestMirrorIndexThings:
         result = mapp.getreleaseslist("pkg")
         assert len(result) == 1
         simpypi.remove_project('pkg')
-        indexconfig['mirror_cache_expiry'] = 0
-        mapp.modify_index("mirror7/mirror", indexconfig=indexconfig)
         result = mapp.getreleaseslist("pkg")
         # serving stale links indefinitely
         # we can't explicitly test for that here, because these tests also run
