@@ -4,12 +4,14 @@ import py
 import pytest
 import re
 import sys
+import tarfile
 from devpi.upload import Checkout
 from devpi.upload import find_parent_subpath
 from devpi.upload import filter_latest
 from devpi.upload import get_pkginfo
 from devpi.upload import main
 from devpi.upload import read_setupcfg
+from io import BytesIO
 from textwrap import dedent
 from devpi_common.metadata import splitbasename
 from devpi_common.viewhelp import ViewLinkStore
@@ -94,7 +96,7 @@ class TestCheckout:
         return repo
 
     def test_vcs_export(self, uploadhub, repo, setupdir, tmpdir, monkeypatch):
-        checkout = Checkout(uploadhub, setupdir)
+        checkout = Checkout(uploadhub, uploadhub.args, setupdir)
         assert checkout.rootpath == repo
         newrepo = tmpdir.mkdir("newrepo")
         result = checkout.export(newrepo)
@@ -113,7 +115,7 @@ class TestCheckout:
     def test_vcs_export_setupdironly(self, uploadhub, setupdir,
                                           tmpdir, monkeypatch):
         monkeypatch.setattr(uploadhub.args, "setupdironly", True)
-        checkout = Checkout(uploadhub, setupdir)
+        checkout = Checkout(uploadhub, uploadhub.args, setupdir)
         assert checkout.rootpath == setupdir
         newrepo = tmpdir.mkdir("newrepo")
         result = checkout.export(newrepo)
@@ -129,7 +131,7 @@ class TestCheckout:
     def test_vcs_export_disabled(self, uploadhub, setupdir,
                                       tmpdir, monkeypatch):
         monkeypatch.setattr(uploadhub.args, "novcs", True)
-        checkout = Checkout(uploadhub, setupdir)
+        checkout = Checkout(uploadhub, uploadhub.args, setupdir)
         assert not checkout.hasvcs
         exported = checkout.export(tmpdir)
         assert exported.rootpath == checkout.setupdir
@@ -138,14 +140,13 @@ class TestCheckout:
                                           tmpdir, monkeypatch):
         subdir = setupdir.mkdir("subdir")
         subdir.ensure("setup.py")
-        checkout = Checkout(uploadhub, subdir)
+        checkout = Checkout(uploadhub, uploadhub.args, subdir)
         wc = tmpdir.mkdir("wc")
         exported = checkout.export(wc)
-        with pytest.raises(SystemExit):
-            exported.check_setup()
+        assert not exported.rootpath.join("setup.py").check()
 
     def test_export_attributes(self, uploadhub, setupdir, tmpdir, monkeypatch):
-        checkout = Checkout(uploadhub, setupdir)
+        checkout = Checkout(uploadhub, uploadhub.args, setupdir)
         setupdir.join("setup.py").write(dedent("""
             from setuptools import setup
             # some packages like numpy produce output during build, simulate:
@@ -158,7 +159,7 @@ class TestCheckout:
         assert version == "1.2.3"
 
     def test_setup_build_docs(self, uploadhub, setupdir, tmpdir, monkeypatch):
-        checkout = Checkout(uploadhub, setupdir)
+        checkout = Checkout(uploadhub, uploadhub.args, setupdir)
         setupdir.join("setup.py").write(dedent("""
             from setuptools import setup
             setup(name="xyz", version="1.2.3")
@@ -172,7 +173,7 @@ class TestCheckout:
         old_popen_output = exported.hub.popen_output
 
         def mock_popen_output(args, **kwargs):
-            if "build_sphinx" in args:
+            if "sphinx-build" in args:
                 l.append(kwargs)
             else:
                 return old_popen_output(args, **kwargs)
@@ -231,10 +232,16 @@ def test_post_includes_auth_info(initproj, monkeypatch, uploadhub):
 
     class args:
         dryrun = None
+        formats = "sdist,bdist_wheel"
         index = None
+        no_isolation = True
+        novcs = None
         only_latest = None
         onlydocs = None
         path = None
+        python = None
+        setupdironly = None
+        verbose = 0
         withdocs = None
 
     initproj("pkg-1.0")
@@ -250,62 +257,286 @@ def test_post_includes_auth_info(initproj, monkeypatch, uploadhub):
     uploadhub.current.set_basic_auth("basic", "auth")
     uploadhub.current.set_client_cert(certpath)
     main(uploadhub, args)
-    (submit, upload) = Session.posts
-    assert submit[0][1][":action"] == "submit"
-    assert submit[1]["auth"] == ("basic", "auth")
-    assert submit[1]["cert"] == certpath
-    assert "X-Devpi-Auth" in submit[1]["headers"]
-    assert upload[0][1][":action"] == "file_upload"
-    assert upload[1]["auth"] == ("basic", "auth")
-    assert upload[1]["cert"] == certpath
-    assert "X-Devpi-Auth" in upload[1]["headers"]
+    (upload1, upload2) = Session.posts
+    assert upload1[0][1][":action"] == "file_upload"
+    assert upload1[1]["auth"] == ("basic", "auth")
+    assert upload1[1]["cert"] == certpath
+    assert "X-Devpi-Auth" in upload1[1]["headers"]
+    assert upload2[0][1][":action"] == "file_upload"
+    assert upload2[1]["auth"] == ("basic", "auth")
+    assert upload2[1]["cert"] == certpath
+    assert "X-Devpi-Auth" in upload2[1]["headers"]
+
+
+@pytest.mark.skipif("sys.version_info < (3, 6)")
+@pytest.mark.skipif("config.option.fast")
+def test_post_derived_devpi_token(initproj, monkeypatch, uploadhub):
+    from base64 import b64decode
+    import pypitoken
+
+    class Session:
+        posts = []
+
+        def post(self, *args, **kwargs):
+            class reply:
+                status_code = 200
+            self.posts.append((args, kwargs))
+            return reply
+
+    class args:
+        dryrun = None
+        formats = None
+        index = None
+        no_isolation = True
+        novcs = None
+        only_latest = None
+        onlydocs = None
+        path = None
+        python = None
+        sdist = False
+        setupdironly = None
+        withdocs = None
+        verbose = 0
+        wheel = True
+
+    initproj("pkg-1.0")
+    passwd = "devpi-AgEAAhFmc2NodWx6ZS1yTlk5a0RuYQAABiBcjsOFkn7_3fn6mFoeJve_cOv-thDRL-4fQzbf_sOGjQ"
+    token = pypitoken.token.Token.load(passwd)
+    assert pypitoken.token.ProjectsRestriction(
+        projects=["pkg"]) not in token.restrictions
+    tmpdir = py.path.local()
+    uploadhub.cwd = tmpdir
+    uploadhub.http = Session()
+    uploadhub.current.reconfigure(dict(
+        index="http://devpi/foo/bar",
+        login="http://devpi/+login",
+        pypisubmit="http://devpi/foo/bar"))
+    uploadhub.current.set_auth("devpi", passwd)
+    main(uploadhub, args)
+    ((post_args, post_kwargs),) = Session.posts
+    auth = post_kwargs['headers']['X-Devpi-Auth']
+    (username, derived_passwd) = (
+        x.decode('ascii') for x in b64decode(auth).split(b':'))
+    assert username == 'devpi'
+    assert derived_passwd != passwd
+    derived_token = pypitoken.token.Token.load(derived_passwd)
+    assert pypitoken.token.ProjectsRestriction(
+        projects=["pkg"]) in derived_token.restrictions
 
 
 class TestUploadFunctional:
-    @pytest.mark.parametrize("projname_version", [
-        "hello-1.0", "my-pkg-123-1.0"])
-    def test_all(self, initproj, devpi, out_devpi, projname_version):
-        initproj(projname_version.rsplit("-", 1), {"doc": {
-            "conf.py": "#nothing",
-            "contents.rst": "",
-            "index.html": "<html/>"}})
+    @pytest.fixture(params=["hello-1.0", "my-pkg-123-1.0"])
+    def projname_version(self, request, initproj):
+        initproj(
+            request.param.rsplit("-", 1),
+            {
+                "doc" if request.param.startswith("hello") else "docs":
+                {
+                    "conf.py": "#nothing",
+                    "contents.rst": "",
+                    "index.html": "<html/>"
+                },
+            }
+        )
+        return request.param
+
+    def test_plain_dry_run(self, devpi, out_devpi, projname_version):
         assert py.path.local("setup.py").check()
-        out = out_devpi("upload", "--dry-run")
+        out = out_devpi("upload", "--no-isolation", "--dry-run")
         assert out.ret == 0
         out.stdout.fnmatch_lines("""
             built:*
             skipped: file_upload of {projname_version}.*
             """.format(projname_version=projname_version))
-        out = out_devpi("upload", "--dry-run", "--with-docs")
+
+    def test_with_docs_dry_run(self, devpi, out_devpi, projname_version):
+        out = out_devpi("upload", "--no-isolation", "--dry-run", "--with-docs")
         assert out.ret == 0
         out.stdout.fnmatch_lines("""
             built:*
             skipped: file_upload of {projname_version}.*
             skipped: doc_upload of {projname_version}.doc.zip*
             """.format(projname_version=projname_version))
-        out = out_devpi("upload", "--dry-run", "--only-docs")
+
+    def test_only_docs_dry_run(self, devpi, out_devpi, projname_version):
+        out = out_devpi("upload", "--no-isolation", "--dry-run", "--only-docs")
         assert out.ret == 0
         out.stdout.fnmatch_lines("""
             built:*
             skipped: doc_upload of {projname_version}.doc.zip*
             """.format(projname_version=projname_version))
-        out = out_devpi("upload", "--with-docs", code=[200,200,200])
+
+    @pytest.mark.parametrize("path", [
+        "foo.doc.zip",
+        "foo.docs.zip"])
+    def test_only_docs_with_path_no_version(self, devpi, out_devpi, path, tmpdir):
+        archive_path = tmpdir.join(path)
+        archive_path.ensure()
+        tmpdir.chdir()
+        out = out_devpi("upload", "--no-isolation", "--only-docs", path)
+        assert out.ret == 1
+        out.stdout.fnmatch_lines(
+            "doczip has no version and 'foo' has no releases to derive one from")
+
+    @pytest.mark.parametrize("path", [
+        "foo.doc.tar.gz",
+        "foo.docs.tgz"])
+    def test_only_docs_with_path_no_version_gz(self, devpi, out_devpi, path, tmpdir):
+        archive_path = tmpdir.join(path)
+        with tarfile.TarFile(archive_path.strpath, "w") as tgz:
+            tgz.addfile(tarfile.TarInfo("index.html"), BytesIO(b""))
+        tmpdir.chdir()
+        out = out_devpi("upload", "--no-isolation", "--only-docs", path)
+        assert out.ret == 1
+        out.stdout.fnmatch_lines("""
+            repackaged {path} to foo.doc.zip
+            doczip has no version and 'foo' has no releases to derive one from
+            """.format(path=path))
+
+    @pytest.mark.parametrize("path", [
+        "foo-1.0.doc.zip",
+        "foo-1.0.docs.zip"])
+    def test_only_docs_with_path(self, devpi, out_devpi, path, tmpdir):
+        archive_path = tmpdir.join(path)
+        archive_path.ensure()
+        tmpdir.chdir()
+        out = out_devpi("upload", "--no-isolation", "--only-docs", path)
+        assert out.ret == 0
+        out.stdout.fnmatch_lines("""
+            doc_upload of {path}*
+            """.format(path=path))
+
+    @pytest.mark.parametrize("path", [
+        "foo-1.0.doc.tar.gz",
+        "foo-1.0.docs.tgz"])
+    def test_only_docs_with_path_gz(self, devpi, out_devpi, path, tmpdir):
+        archive_path = tmpdir.join(path)
+        with tarfile.TarFile(archive_path.strpath, "w") as tgz:
+            tgz.addfile(tarfile.TarInfo("index.html"), BytesIO(b""))
+        tmpdir.chdir()
+        out = out_devpi("upload", "--no-isolation", "--only-docs", path)
+        assert out.ret == 0
+        out.stdout.fnmatch_lines("""
+            repackaged {path} to foo-1.0.doc.zip
+            doc_upload of foo-1.0.doc.zip*
+            """.format(path=path))
+
+    def test_plain_with_docs(self, devpi, out_devpi, projname_version):
+        out = out_devpi("upload", "--no-isolation", "--with-docs", code=[200, 200, 200])
         assert out.ret == 0
         out.stdout.fnmatch_lines("""
             built:*
             file_upload of {projname_version}.*
             doc_upload of {projname_version}.doc.zip*
             """.format(projname_version=projname_version))
-        out = out_devpi("upload", "--formats", "sdist.zip", code=[200,200])
+
+    def test_sdist_zip_with_docs(self, devpi, out_devpi, projname_version):
+        out = out_devpi(
+            "upload", "--formats", "sdist.zip", "--with-docs", code=[200, 200])
+        assert out.ret == 0
+        out.stdout.fnmatch_lines("""
+            built:*
+            file_upload of {projname_version}.zip*
+            doc_upload of {projname_version}.doc.zip*
+            """.format(projname_version=projname_version))
+
+    def test_sdist_zip(self, devpi, out_devpi, projname_version):
+        out = out_devpi("upload", "--no-isolation", "--formats", "sdist.zip", code=[200])
         assert out.ret == 0
         out.stdout.fnmatch_lines("""
             built:*
             file_upload of {projname_version}.zip*
             """.format(projname_version=projname_version))
 
-        print("*"*80)
-        out = out_devpi("upload", "--formats", "sdist.zip,bdist_wheel",
-                        code=[200, 200, 200, 200])
+    def test_sdist(self, devpi, out_devpi, projname_version):
+        out = out_devpi("upload", "--no-isolation", "--sdist", code=[200])
+        assert out.ret == 0
+        out.stdout.fnmatch_lines("""
+            built:*
+            file_upload of {projname_version}*
+            """.format(projname_version=projname_version))
+
+    def test_native_sdist(self, devpi, out_devpi, projname_version):
+        if sys.platform == "win32":
+            nativeformat = "zip"
+            nativeext = ".zip"
+        else:
+            nativeformat = "tgz"
+            nativeext = ".tar.gz"
+        out = out_devpi("upload", "--no-isolation", "--formats", "sdist.%s" % nativeformat, code=[200])
+        assert out.ret == 0
+        out.stdout.fnmatch_lines("""
+            The --formats option is deprecated, replace it with --sdist to only*
+            built:*
+            file_upload of {projname_version}{nativeext}*
+            """.format(projname_version=projname_version, nativeext=nativeext))
+
+    def test_bdist_wheel(self, devpi, out_devpi, projname_version):
+        out = out_devpi("upload", "--no-isolation", "--formats", "bdist_wheel", code=[200])
+        assert out.ret == 0
+        projname_version_norm = projname_version.replace("-", "*")
+        out.stdout.fnmatch_lines("""
+            The --formats option is deprecated, replace it with --wheel to only*
+            built:*
+            file_upload of {projname_version_norm}*.whl*
+            """.format(projname_version_norm=projname_version_norm))
+
+    def test_wheel_setup_cfg(self, devpi, initproj, out_devpi):
+        initproj("pkg-1.0", kind="setup.cfg")
+        out = out_devpi("upload", "--no-isolation", "--wheel", code=[200])
+        assert out.ret == 0
+        out.stdout.fnmatch_lines("""
+            built:*
+            file_upload of pkg-1.0-*.whl*
+            """)
+
+    @pytest.mark.skipif("sys.version_info < (3,)")
+    def test_wheel_pyproject_toml(self, devpi, initproj, out_devpi):
+        initproj("pkg-1.0", kind="pyproject.toml")
+        out = out_devpi("upload", "--wheel", code=[200])
+        assert out.ret == 0
+        out.stdout.fnmatch_lines("""
+            built:*
+            file_upload of pkg-1.0-*.whl*
+            """)
+
+    def test_default_formats(self, devpi, out_devpi, projname_version):
+        if sys.platform == "win32":
+            nativeext = ".zip"
+        else:
+            nativeext = ".tar.gz"
+        out = out_devpi(
+            "upload", "--formats", "sdist,bdist_wheel", code=[200, 200])
+        assert out.ret == 0
+        projname_version_norm = projname_version.replace("-", "*")
+        out.stdout.fnmatch_lines_random("""
+            The --formats option is deprecated, you can remove it to get the*
+            built:*
+            file_upload of {projname_version_norm}*.whl*
+            file_upload of {projname_version}{nativeext}*
+            """.format(
+            projname_version=projname_version,
+            projname_version_norm=projname_version_norm,
+            nativeext=nativeext))
+
+    def test_deprecated_formats(self, devpi, out_devpi, projname_version):
+        out = out_devpi(
+            "upload", "--formats", "bdist_dumb,bdist_egg", code=[200, 200])
+        assert out.ret == 0
+        projname_version_norm = projname_version.replace("-", "*")
+        out.stdout.fnmatch_lines_random("""
+            The --formats option is deprecated, none of the specified formats 'bdist_dumb,bdist_egg'*
+            *Falling back to 'setup.py bdist_dumb' which*
+            *Falling back to 'setup.py bdist_egg' which*
+            built:*
+            file_upload of {projname_version}*.tar.gz*
+            file_upload of {projname_version_norm}*.egg*
+            """.format(
+            projname_version=projname_version,
+            projname_version_norm=projname_version_norm))
+
+    def test_plain(self, devpi, out_devpi, projname_version):
+        out = out_devpi("upload", "--no-isolation", code=[200, 200])
         out.stdout.fnmatch_lines_random("""
             file_upload of {projname_version}.*
             file_upload of {projname_version_norm}*.whl*
@@ -313,32 +544,51 @@ class TestUploadFunctional:
                        projname_version_norm=projname_version.replace("-", "*")
                        ))
 
+    def test_upload_to_mirror(
+            self, devpi, initproj, out_devpi, projname_version):
+        initproj(projname_version.rsplit("-", 1), {"doc": {
+            "conf.py": "#nothing",
+            "contents.rst": "",
+            "index.html": "<html/>"}})
+        assert py.path.local("setup.py").check()
+
+        # use mirror
+        out = out_devpi("use", "root/pypi")
+        out.stdout.fnmatch_lines_random("current devpi index*/root/pypi*")
+        out = out_devpi("upload", "--no-isolation", "--dry-run")
+        out.stdout.fnmatch_lines_random("*does not support upload.")
+        out.stdout.fnmatch_lines_random("*it is a mirror.")
+
+    @pytest.mark.parametrize("other_index", ["root/pypi", "/"])
+    def test_index_option(
+            self, devpi, initproj, out_devpi, other_index, projname_version):
+        initproj(projname_version.rsplit("-", 1), {"doc": {
+            "conf.py": "#nothing",
+            "contents.rst": "",
+            "index.html": "<html/>"}})
+        assert py.path.local("setup.py").check()
         # remember username
         out = out_devpi("use")
         user = re.search(r'\(logged in as (.+?)\)', out.stdout.str()).group(1)
 
         # go to other index
-        out = out_devpi("use", "root/pypi")
-        out.stdout.fnmatch_lines_random("current devpi index*/root/pypi*")
-        out = out_devpi("upload", "--dry-run")
-        out.stdout.fnmatch_lines_random("*does not support upload.")
-        out.stdout.fnmatch_lines_random("*it is a mirror.")
+        out = out_devpi("use", other_index)
 
         # --index option
-        out = out_devpi("upload", "--index", "%s/dev" % user, "--dry-run")
-        out.stdout.fnmatch_lines_random("skipped: register*to*/%s/dev*" % user)
+        out = out_devpi("upload", "--no-isolation", "--index", "%s/dev" % user, "--dry-run")
+        out.stdout.fnmatch_lines_random("skipped: file_upload*to*/%s/dev*" % user)
 
-        # go back
-        out = out_devpi("use", "%s/dev" % user)
-        out.stdout.fnmatch_lines_random("current devpi index*/%s/dev*" % user)
+    def test_logout(self, capfd, devpi, out_devpi, projname_version):
         # logoff then upload
         out = out_devpi("logoff")
         out.stdout.fnmatch_lines_random("login information deleted")
-        out = out_devpi("upload", "--dry-run")
-        out.stdout.fnmatch_lines_random("need to be authenticated*")
 
         # see if we get an error return code
-        res = devpi("upload")
+        (out, err) = capfd.readouterr()
+        res = devpi("upload", "--no-isolation")
+        (out, err) = capfd.readouterr()
+        assert "401 FAIL file_upload" in out
+        assert "Unauthorized" in out
         assert isinstance(res.sysex, SystemExit)
         assert res.sysex.args == (1,)
 
@@ -352,7 +602,7 @@ class TestUploadFunctional:
         runproc(tmpdir, "python setup.py sdist --format=zip".split())
         dist = tmpdir.join("dist")
         assert len(dist.listdir()) == 2
-        hub = devpi("upload", "--from-dir", dist)
+        hub = devpi("upload", "--no-isolation", "--from-dir", dist)
         for ver in ("1.1", '1.2'):
             url = hub.current.get_index_url().url + "hello/%s/" % ver
             out = out_devpi("getjson", url)
@@ -384,7 +634,7 @@ class TestUploadFunctional:
         zip_dir(bpath.join('html'), dist.join("%s.doc.zip" % name_version_str))
         assert len(dist.listdir()) == 2
         (p, dp) = sorted(dist.listdir(), key=lambda x: '.doc.zip' in x.basename)
-        hub = devpi("upload", p, dp)
+        hub = devpi("upload", "--no-isolation", p, dp)
         url = hub.current.get_index_url().url + path
         out = out_devpi("getjson", url)
         data = json.loads(out.stdout.str())
